from enum import StrEnum
from typing import Any


class AlignmentTags(StrEnum):
    BAM: str = "bam"
    BAM_INDEX: str = "bam-index"
    BAM_MT: str = "bam-mt"
    BAM_MT_INDEX: str = "bam-mt-index"
    CRAM: str = "cram"
    CRAM_INDEX: str = "cram-index"

    @classmethod
    def name(cls) -> str:
        return "Alignment Tags"

    @property
    def description(self) -> str:
        descriptions: dict[AlignmentTags, str] = {
            self.BAM: "Alignment file in BAM format",
            self.BAM_INDEX: "Index file for alignment file in BAM format",
            self.BAM_MT: "Alignment file in BAM format holding the reads from chrMT",
            self.CRAM: "Alignment file in CRAM format",
            self.CRAM_INDEX: "Index file for alignment file in CRAM format",
        }
        return descriptions.get(self, "Description not available")


class RawDataTags(StrEnum):
    FASTA: str = "fasta"
    FASTQ: str = "fastq"
    FORWARD_STRAND: str = "forward-strand"
    REVERSE_STRAND: str = "reverse-strand"
    RNA: str = "rna"
    UNPAIRED_READS: str = "unpaired-reads"

    @classmethod
    def name(cls) -> str:
        return "Raw Data Tags"

    @property
    def description(self) -> str:
        descriptions: dict[RawDataTags, str] = {
            self.FASTA: "Files with raw data in fasta format",
            self.FASTQ: "Files with raw data in fastq format",
            self.FORWARD_STRAND: "Reads from forward strand",
            self.REVERSE_STRAND: "Reads from reverse strand",
            self.RNA: "Data generated from RNA samples",
            self.UNPAIRED_READS: "Reads that could not be paired",
        }
        return descriptions.get(self, "Description not available")


class VariantTags(StrEnum):
    CNV: str = "cnv"
    GERMLINE: str = "germline"
    MOBILE_ELEMENTS: str = "mobile-elements"
    MSI: str = "msi"
    NORMAL: str = "normal"
    RHOCALL_VIZ: str = "rhocall-viz"
    SMN_CALLING: str = "smn-calling"
    SNV: str = "snv"
    SNV_BCF: str = "snv-bcf"
    SNV_GBCF: str = "snv-gbcf"
    SOMATIC: str = "somatic"
    SV: str = "sv"
    SV_BCF: str = "sv-bcf"
    SV_VCF: str = "sv-vcf"
    SV_VCF_INDEX: str = "sv-vcf-index"
    TELOMERE_CALLING: str = "telomere-calling"
    TMB: str = "tmb"
    TUMOR: str = "tumor"
    UPD: str = "upd"
    VARIANTS: str = "variants"
    VCF2CYTOSURE: str = "vcf2cytosure"
    VCF: str = "vcf"
    VCF_INDEX: str = "vcf-index"
    VCF_SNV: str = "vcf-snv"
    VCF_SNV_CLINICAL: str = "vcf-snv-clinical"
    VCF_SNV_CLINICAL_INDEX: str = "vcf-snv-clinical-index"
    VCF_SNV_CLINICAL_SCORED: str = "vcf-snv-clinical-scored"
    VCF_SNV_CLINICAL_SCORED_INDEX: str = "vcf-snv-clinical-scored-index"
    VCF_SNV_FILTERED: str = "vcf-snv-filtered"
    VCF_SNV_FILTERED_INDEX: str = "vcf-snv-filtered-index"
    VCF_SNV_GERMLINE_NORMAL: str = "vcf-snv-germline-normal"
    VCF_SNV_GERMLINE_NORMAL_INDEX: str = "vcf-snv-germline-normal-index"
    VCF_SNV_GERMLINE_TUMOR: str = "vcf-snv-germline-tumor"
    VCF_SNV_GERMLINE_TUMOR_INDEX: str = "vcf-snv-germline-tumor-index"
    VCF_SNV_INDEX: str = "vcf-snv-index"
    VCF_SNV_RESEARCH: str = "vcf-snv-research"
    VCF_SNV_RESEARCH_INDEX: str = "vcf-snv-research-index"
    VCF_SNV_RESEARCH_UNFILTERED: str = "vcf-snv-research-unfiltered"
    VCF_SNV_RESEARCH_UNFILTERED_INDEX: str = "vcf-snv-research-unfiltered-index"
    VCF_STR: str = "vcf-str"
    VCF_STR_INDEX: str = "vcf-str-index"
    VCF_SV: str = "vcf-sv"
    VCF_SV_CLINICAL: str = "vcf-sv-clinical"
    VCF_SV_CLINICAL_INDEX: str = "vcf-sv-clinical-index"
    VCF_SV_GERMLINE_NORMAL: str = "vcf-sv-germline-normal"
    VCF_SV_GERMLINE_NORMAL_INDEX: str = "vcf-sv-germline-normal-index"
    VCF_SV_GERMLINE_TUMOR: str = "vcf-sv-germline-tumor"
    VCF_SV_GERMLINE_TUMOR_INDEX: str = "vcf-sv-germline-tumor-index"
    VCF_SV_INDEX: str = "vcf-sv-index"
    VCF_SV_RESEARCH: str = "vcf-sv-research"
    VCF_SV_RESEARCH_INDEX: str = "vcf-sv-research-index"
    VCF_UMI_SNV: str = "vcf-umi-snv"
    VCF_UMI_SNV_CLINICAL: str = "vcf-umi-snv-clinical"
    VCF_UMI_SNV_CLINICAL_INDEX: str = "vcf-umi-snv-clinical-index"
    VCF_UMI_SNV_INDEX: str = "vcf-umi-snv-index"
    VCF_UMI_SNV_RESEARCH: str = "vcf-umi-snv-research"
    VCF_UMI_SNV_RESEARCH_INDEX: str = "vcf-umi-snv-research-index"
    VCF_UMI_SNV_RESEARCH_UNFILTERED: str = "vcf-umi-snv-research-unfiltered"
    VCF_UMI_SNV_RESEARCH_UNFILTERED_INDEX: str = "vcf-umi-snv-research-unfiltered-index"
    VCF_UMI_SNV_CLINICAL_RANKED: str = "vcf-umi-snv-clinical-scored"
    VCF_UMI_SNV_CLINICAL_RANKED_INDEX: str = "vcf-umi-snv-clinical-scored-index"

    @classmethod
    def name(cls) -> str:
        return "Variant Tags"

    @property
    def description(self) -> str:
        descriptions: dict[VariantTags, str] = {
            self.CNV: "Copy number variants",
            self.GERMLINE: "Associated with germline variants",
            self.MOBILE_ELEMENTS: "Mobile elements",
            self.NORMAL: "Associated with normal sample",
            self.RHOCALL_VIZ: "Runs of homozygosity index",
            self.SMN_CALLING: "Copy number calls for the SMN gene",
            self.SNV: "Single nucleotide variants and short indels",
            self.SNV_BCF: "Gvcf including all SNV variants",
            self.SNV_GBCF: "Gvcf including all SNV variants",
            self.SOMATIC: "Associated with somatic variants",
            self.SV: "Structural variants",
            self.SV_BCF: "Gvcf including all SV variants",
            self.SV_VCF: "Variant call formatted file with structural variants",
            self.SV_VCF_INDEX: "Following index",
            self.TELOMERE_CALLING: "Variants from telomere calling",
            self.TMB: "Tumor mutational burden information",
            self.TUMOR: "Associated with tumor sample",
            self.UPD: "Uniparental disomy variants",
            self.VARIANTS: "File pertaining variant information in some way",
            self.VCF2CYTOSURE: "Conversion from VCF format to cytosure format",
            self.VCF: "Variant call formatted file",
            self.VCF_INDEX: "Following index",
            self.VCF_SNV: "SNV variant call formatted file",
            self.VCF_SNV_CLINICAL: "SNV variants from clinical panels",
            self.VCF_SNV_CLINICAL_INDEX: "Following index",
            self.VCF_SNV_CLINICAL_SCORED: "Scored clinically annotated VCF with SNVs",
            self.VCF_SNV_CLINICAL_SCORED_INDEX: "Following index",
            self.VCF_SNV_FILTERED: "SNV variants filtered by BALSAMIC",
            self.VCF_SNV_FILTERED_INDEX: "Following index",
            self.VCF_SNV_GERMLINE_NORMAL: "SNV germline normal variants",
            self.VCF_SNV_GERMLINE_NORMAL_INDEX: "Following index",
            self.VCF_SNV_GERMLINE_TUMOR: "SNV germline tumor variants",
            self.VCF_SNV_GERMLINE_TUMOR_INDEX: "Following index",
            self.VCF_SNV_INDEX: "Following index",
            self.VCF_SNV_RESEARCH: "SNV variants from whole genome",
            self.VCF_SNV_RESEARCH_INDEX: "Following index",
            self.VCF_STR: "Short tandem repeat variants",
            self.VCF_STR_INDEX: "Following index",
            self.VCF_SV: "SV variant call formatted file",
            self.VCF_SV_CLINICAL: "SV variants from clinical panels",
            self.VCF_SV_CLINICAL_INDEX: "Following index",
            self.VCF_SV_GERMLINE_NORMAL: "SV germline normal variants",
            self.VCF_SV_GERMLINE_NORMAL_INDEX: "Following index",
            self.VCF_SV_GERMLINE_TUMOR: "SV germline tumor variants",
            self.VCF_SV_GERMLINE_TUMOR_INDEX: "Following index",
            self.VCF_SV_INDEX: "Following index",
            self.VCF_SV_RESEARCH: "SV variants from whole genome",
            self.VCF_SV_RESEARCH_INDEX: "Following index",
            self.VCF_UMI_SNV: "Raw SNV UMI variant formatted file",
            self.VCF_UMI_SNV_CLINICAL: "SNV UMI variants from clinical panels",
            self.VCF_UMI_SNV_CLINICAL_INDEX: "Following index",
            self.VCF_UMI_SNV_INDEX: "Following index",
            self.VCF_UMI_SNV_RESEARCH: "SNV UMI variant formatted file",
            self.VCF_UMI_SNV_RESEARCH_INDEX: "Following index",
            self.VCF_UMI_SNV_CLINICAL_RANKED: "Scored clinically annotated VCF with SNVs from UMI analysis",
            self.VCF_UMI_SNV_CLINICAL_RANKED_INDEX: "Following index",
        }
        return descriptions.get(self, "Description not available")


class FamilyTags(StrEnum):
    PED: str = "ped"
    PEDIGREE: str = "pedigree"

    @classmethod
    def name(cls) -> str:
        return "Family Tags"

    @property
    def description(self) -> str:
        descriptions: dict[FamilyTags, str] = {
            self.PED: "Pedigree information",
            self.PEDIGREE: "Pedigree information",
        }
        return descriptions.get(self, "Description not available")


class ReportTags(StrEnum):
    AUDIT: str = "audit"
    BCFTOOLS_STATS: str = "bcftools-stats"
    CNV_REPORT: str = "cnv-report"
    CSV: str = "csv"
    D4: str = "d4"
    DELIVERABLE: str = "deliverable"
    DELIVERY_REPORT: str = "delivery-report"
    GENE_COUNTS: str = "gene-counts"
    GENERAL_STATS: str = "general-stats"
    MOSDEPTH_COVDIST: str = "mosdepth-covdist"
    MOSDEPTH_CUMCOV: str = "mosdepth-cumcov"
    MOSDEPTH_PERCHROM: str = "mosdepth-perchrom"
    MULTIQC: str = "multiqc"
    MULTIQC_HTML: str = "multiqc-html"
    MULTIQC_JSON: str = "multiqc-json"
    PDF: str = "pdf"
    PICARD_ALIGNMENT: str = "picard-alignment"
    PICARD_BAMQC: str = "picard-bamqc"
    PICARD_DUPLICATES: str = "picard-duplicates"
    PICARD_HISTOGRAM: str = "picard-histogram"
    PICARD_HS: str = "picard-hs"
    PICARD_INSERT_SIZE: str = "picard-insert-size"
    PICARD_QUALITYCYCLE: str = "picard-qualitycycle"
    PICARD_QUALITYDISTR: str = "picard-qualitydistr"
    PICARD_RNASEQ: str = "picard-rnaseq"
    PICARD_WGS: str = "picard-wgs"
    QC_REPORT: str = "qc-report"
    SAMBAMBA_DEPTH: str = "sambamba-depth"
    SAMTOOLS_STATS: str = "samtools-stats"
    SOFTWARE_VERSIONS: str = "software-versions"
    STAR: str = "star"
    SUMMARY: str = "summary"
    TSV: str = "tsv"
    VCF_REPORT: str = "vcf-report"

    @classmethod
    def name(cls) -> str:
        return "Report Tags"

    @property
    def description(self) -> str:
        descriptions: dict[ReportTags, str] = {
            self.AUDIT: "Audit file",
            self.BCFTOOLS_STATS: "BCFtools variant calling metrics",
            self.CNV_REPORT: "CNV variant calling report",
            self.CSV: "Comma separated values",
            self.D4: "D4 file format coverage file",
            self.DELIVERABLE: "Deliverables file",
            self.DELIVERY_REPORT: "Delivery report with result for upload to Scout",
            self.GENE_COUNTS: "STAR read counts output per gene",
            self.GENERAL_STATS: "General statistics reported from MultiQC",
            self.MOSDEPTH_COVDIST: "Coverage distribution profile from mosdepth",
            self.MOSDEPTH_CUMCOV: "Cumulative coverage distribution profile from mosdepth",
            self.MOSDEPTH_PERCHROM: "Depth per chromosome from mosdepth",
            self.MULTIQC: "MultiQC related files",
            self.MULTIQC_HTML: "MultiQC analysis report in HTML format",
            self.MULTIQC_JSON: "MultiQC analysis report in JSON format",
            self.PDF: "Portable document format",
            self.PICARD_ALIGNMENT: "High level metrics about the alignment of reads",
            self.PICARD_BAMQC: "Qualimap BAMQC metrics",
            self.PICARD_DUPLICATES: "Metrics calculated during marking duplicates",
            self.PICARD_HISTOGRAM: "Picard histograms in JSON format",
            self.PICARD_HS: "Metrics for the analysis of target-capture sequencing",
            self.PICARD_INSERT_SIZE: "Metrics about the insert size distribution",
            self.PICARD_QUALITYCYCLE: "Picard MeanQualityByCycle Metrics",
            self.PICARD_RNASEQ: "Metrics about the alignment of RNA-seq reads",
            self.PICARD_WGS: "Metrics for evaluating the performance of WGS analysis",
            self.QC_REPORT: "Results and QC",
            self.SAMBAMBA_DEPTH: "Coverage information from sambamba",
            self.SAMTOOLS_STATS: "Comprehensive statistics from alignment file",
            self.SOFTWARE_VERSIONS: "Versions of software used in analysis",
            self.STAR: "Files related to Star aligner",
            self.SUMMARY: "Overview file without detailed information",
            self.TSV: "Tab separated values",
            self.VCF_REPORT: "Results and QC from variant calling",
        }
        return descriptions.get(self, "Description not available")


class QCTags(StrEnum):
    PED_CHECK: str = "ped-check"
    QC_METRICS: str = "qc-metrics"
    SEX_CHECK: str = "sex-check"

    @classmethod
    def name(cls) -> str:
        return "QC Tags"

    @property
    def description(self) -> str:
        descriptions: dict[QCTags, str] = {
            self.PED_CHECK: "Results from pedigree validation",
            self.QC_METRICS: "QC metrics from analysis",
            self.SEX_CHECK: "Results from sex validation",
        }
        return descriptions.get(self, "Description not available")


class AnalysisTags(StrEnum):
    ASPCF_PLOT: str = "aspcf-plot"
    ASSEMBLY: str = "assembly"
    AUTOZYG: str = "autozyg"
    BED: str = "bed"
    BED_INDEX: str = "bed-index"
    BIGWIG: str = "bigwig"
    CIRCULAR_PLOT: str = "circular-plot"
    CLINICAL: str = "clinical"
    CONFIG: str = "config"
    COVERAGE: str = "coverage"
    FILTERED: str = "filtered"
    FIRST_PASS: str = "first-pass"
    FRACSNP: str = "fracsnp"
    FUSION: str = "fusion"
    GENES: str = "genes"
    GERMLINE_PLOT: str = "germline-plot"
    JUNCTION: str = "junction"
    METRICS: str = "metrics"
    MITOCHONDRIA: str = "mitochondria"
    PROFILE_PLOT: str = "profile-plot"
    QC_CRAM: str = "qc-cram"
    QC_CRAM_INDEX: str = "qc-cram-index"
    RAW_PROFILE_PLOT: str = "raw-profile-plot"
    REFERENCE_INFO: str = "reference-info"
    REGIONS: str = "regions"
    RESEARCH: str = "research"
    SCATTER_PLOT: str = "scatter-plot"
    SECOND_PASS: str = "second-pass"
    SEGMENTS: str = "segments"
    SITES: str = "sites"
    SUNRISE_PLOT: str = "sunrise-plot"
    TCOV: str = "tcov"
    TIDDIT_COVERAGE: str = "tiddit-coverage"
    TUMOR_PLOT: str = "tumor-plot"
    UMI: str = "umi"
    UMI_CRAM: str = "umi-cram"
    UMI_CRAM_INDEX: str = "umi-cram-index"
    VCF_FUSION: str = "vcf-fusion"
    VISUALIZATION: str = "visualization"

    @classmethod
    def name(cls) -> str:
        return "Analysis Tags"

    @property
    def description(self) -> str:
        descriptions: dict[AnalysisTags, str] = {
            self.ASPCF_PLOT: "Plot of LogR and BAF values",
            self.ASSEMBLY: "Assembly",
            self.AUTOZYG: "Autozygous region",
            self.BED: "Bed file",
            self.BED_INDEX: "Following index",
            self.BIGWIG: "Bigwig formatted file",
            self.CIRCULAR_PLOT: "Circular plot",
            self.CLINICAL: "Clinical subset",
            self.CONFIG: "Config file",
            self.COVERAGE: "Output with coverage information",
            self.FILTERED: "Filtered data",
            self.FIRST_PASS: "Data metrics first pass",
            self.FRACSNP: "Fraction of reads with SNP",
            self.FUSION: "Fusion transcripts",
            self.GENES: "Related to genes",
            self.GERMLINE_PLOT: "Plot of LogR and BAF values for normal sample",
            self.JUNCTION: "Junction data",
            self.METRICS: "Data metrics",
            self.MITOCHONDRIA: "Mitochondria related",
            self.PROFILE_PLOT: "Copy number profile plot",
            self.QC_CRAM: "QC alignment file in CRAM format",
            self.QC_CRAM_INDEX: "QC index file for alignment file in CRAM format",
            self.RAW_PROFILE_PLOT: "Copy number profile without rounding to whole numbers",
            self.REFERENCE_INFO: "Information about references used in analysis",
            self.REGIONS: "Output for regions",
            self.RESEARCH: "Research set",
            self.SCATTER_PLOT: "Scatter plot",
            self.SECOND_PASS: "Data metrics second pass",
            self.SEGMENTS: "Output for segments",
            self.SITES: "Output for sites",
            self.SUNRISE_PLOT: "Sunrise plot",
            self.TCOV: "Coverage output",
            self.TIDDIT_COVERAGE: "Coverage output from tiddit",
            self.TUMOR_PLOT: "Plot of LogR and BAF values for tumor sample",
            self.UMI: "Files related to UMI workflow",
            self.UMI_CRAM: "UMI consensus filtered cram file",
            self.UMI_CRAM_INDEX: "Index for the UMI consensus filtered cram file",
            self.VCF_FUSION: "Converted RNA fusion file to SV VCF",
            self.VISUALIZATION: "Visualizes data",
        }
        return descriptions.get(self, "Description not available")


class BioinfoToolsTags(StrEnum):
    ARRIBA: str = "arriba"
    ASCATNGS: str = "ascatngs"
    ASEREADCOUNTER: str = "asereadcounter"
    CHANJO: str = "chanjo"
    CHROMOGRAPH: str = "chromograph"
    CNVKIT: str = "cnvkit"
    CNVPYTOR: str = "cnvpytor"
    CYRIUS: str = "cyrius"
    DEEPVARIANT: str = "deepvariant"
    DELLY: str = "delly"
    DESEQ2: str = "deseq2"
    DNASCOP: str = "dnascope"
    EKLIPSE: str = "eklipse"
    EXPANSIONHUNTER: str = "expansionhunter"
    FASTP: str = "fastp"
    FASTQC: str = "fastqc"
    FUSIONCATCHER: str = "fusioncatcher"
    FUSIONCATCHER_SUMMARY: str = "fusioncatcher-summary"
    FUSIONINSPECTOR: str = "fusioninspector"
    GENOTYPER: str = "genotyper"
    GENS: str = "gens"
    GFFCOMPARE: str = "gffcompare"
    HAPLOTYPE_CALLER: str = "haplotype-caller"
    MANTA: str = "manta"
    MERGED: str = "merged"
    MITODEL: str = "mitodel"
    NEXTCLADE: str = "nextclade"
    PEDDY: str = "peddy"
    PICARD: str = "picard"
    PIZZLY: str = "pizzly"
    RETROSEQ: str = "retroseq"
    SALMON_QUANT: str = "salmon-quant"
    SENTION: str = "sention"
    SOMALIER: str = "somalier"
    SQUID: str = "squid"
    STAR_FUSION: str = "star-fusion"
    STRINGTIE: str = "stringtie"
    SVDB: str = "svdb"
    TIDDIT: str = "tiddit"
    TNSCOPE: str = "tnscope"
    TNSCOPE_UMI: str = "tnscope-umi"
    UPD: str = "upd"
    VARDICT: str = "vardict"
    WISECONDOR: str = "wisecondor"

    @classmethod
    def name(cls) -> str:
        return "Bioinfo Tools Tags"

    @property
    def description(self) -> str:
        descriptions: dict[BioinfoToolsTags, str] = {
            self.ARRIBA: "Fusion caller",
            self.ASCATNGS: "Tool to identify somatically acquired copy-number alterations",
            self.ASEREADCOUNTER: "Count reads mapping to heterozygous sites",
            self.CHANJO: "Tool to keep track of coverage over specific regions",
            self.CHROMOGRAPH: "Tool to create PNG images from BED and WIG files from mikaell",
            self.CNVKIT: "Tool to call copy number variations",
            self.CNVPYTOR: "Tool for CNV/CNA analysis from depth-of-coverage by mapped reads",
            self.CYRIUS: "Tool to call the problematic CYP2D6 gene",
            self.DEEPVARIANT: "Variantcaller",
            self.DELLY: "Cancer structural variant prediction tool",
            self.DESEQ2: "Differential expression analysis with DESeq2",
            self.DNASCOP: "Call snv indels",
            self.EKLIPSE: "Detection and quantification of mitochondrial DNA deletions",
            self.EXPANSIONHUNTER: "Call repeat expansions",
            self.FASTP: "Preprocessing tool for FastQ files",
            self.FASTQC: "Quality control tool for high throughput sequence data",
            self.FUSIONCATCHER: "Fusion caller",
            self.FUSIONCATCHER_SUMMARY: "Fusion caller summary",
            self.FUSIONINSPECTOR: "Fusion inspection",
            self.GENOTYPER: "SNV indel caller from sention",
            self.GENS: "CNV visualization tool",
            self.GFFCOMPARE: "Compare gff files",
            self.HAPLOTYPE_CALLER: "Call snv and indels",
            self.MANTA: "Tool to call structural variants",
            self.MITODEL: "Tool to identify mitochondrial deletion signatures",
            self.NEXTCLADE: "Viral genome clade assignment",
            self.PEDDY: "Tool to check pedigree and ancestral relations",
            self.PICARD: "Picard set of bioinformatic tools",
            self.PIZZLY: "Fusion caller",
            self.RETROSEQ: "Mobile element caller",
            self.SALMON_QUANT: "Transcript quantification",
            self.SENTION: "Sention algorithm",
            self.SOMALIER: "Tool for sample-swap and relatedness checks",
            self.SQUID: "Fusion caller",
            self.STAR_FUSION: "Fusion caller",
            self.STRINGTIE: "Transcript assembler",
            self.SVDB: "Tool to merge SV vcf files from multiple variant callers",
            self.TIDDIT: "Tool to identify chromosomal rearrangements",
            self.TNSCOPE: "Call snv indels",
            self.TNSCOPE_UMI: "Call snv indels for umi",
            self.UPD: "Uniparent disomy caller from bjhall",
            self.VARDICT: "Cancer variant caller",
            self.WISECONDOR: "NIPT caller",
        }
        return descriptions.get(self, "Description not available")


class MipTags(StrEnum):
    EXE_VER: str = "exe-ver"
    MIP_ANALYSE: str = "mip-analyse"
    MIP_CONFIG: str = "mip-config"
    MIP_LOG: str = "mip-log"
    PEDIGREE_YAML: str = "pedigree-yaml"
    SAMPLE_INFO: str = "sample-info"
    VARIANT_CATALOG: str = "variant-catalog"

    @classmethod
    def name(cls) -> str:
        return "MIP Tags"

    @property
    def description(self) -> str:
        descriptions: dict[MipTags, str] = {
            self.EXE_VER: "Executable versions",
            self.MIP_ANALYSE: "MIP information about analysis",
            self.MIP_CONFIG: "MIP configs for analysis",
            self.MIP_LOG: "Logs for MIP analysis",
            self.PEDIGREE_YAML: "YAML file with pedigree information",
            self.SAMPLE_INFO: "MIP info file about samples",
            self.VARIANT_CATALOG: "Variant catalog used with expansionhunter",
        }
        return descriptions.get(self, "Description not available")


class BalsamicTags(StrEnum):
    BALSAMIC_CONFIG: str = "balsamic-config"
    BALSAMIC_DAG: str = "balsamic-dag"
    BALSAMIC_REPORT: str = "balsamic-report"

    @classmethod
    def name(cls) -> str:
        return "BALSAMIC Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, BalsamicTags] = {
            self.BALSAMIC_CONFIG: "Balsamic configs for analysis",
            self.BALSAMIC_DAG: "Balsamic run schema",
            self.BALSAMIC_REPORT: "Report from analysis",
        }
        return descriptions.get(self, "Description not available")


class MicrosaltTags(StrEnum):
    MICROSLAT_CONFIG: str = "microsalt-config"
    MICROSLAT_LOG: str = "microsalt-log"
    TYPING_REPORT: str = "typing-report"

    @classmethod
    def name(cls) -> str:
        return "MicroSalt Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, MicrosaltTags] = {
            self.MICROSLAT_CONFIG: "Config settings for microsalt analysis",
            self.MICROSLAT_LOG: "SLURM log for microsalt analysis",
            self.TYPING_REPORT: "Results from bacterial typing",
        }
        return descriptions.get(self, "Description not available")


class MutantTags(StrEnum):
    ARTIC_JSON: str = "artic-json"
    ARTIC_QC: str = "artic-qc"
    ARTIC_SUM: str = "artic-sum"
    ARTIC_TYPE: str = "artic-type"
    ARTIC_VAR: str = "artic-var"
    CONSENSUS: str = "consensus"
    CONSENSUS_SAMPLE: str = "consensus-sample"
    FOHM_DELIVERY: str = "fohm-delivery"
    INSTRUMENT_PROPERTIES: str = "instrument-properties"
    KOMPLETTERING: str = "komplettering"
    KS_AUX_RESULTS: str = "ks-aux-results"
    KS_DELIVERY: str = "ks-delivery"
    KS_RESULTS: str = "ks-results"
    MUTANT_CONFIG: str = "mutant-config"
    MUTANT_LOG: str = "mutant-log"
    PANGOLIN: str = "pangolin"
    PANGOLIN_TYPING: str = "pangolin-typing"
    PANGOLIN_TYPING_FOHM: str = "pangolin-typing-fohm"
    TYPING_REPORT: str = "typing-report"
    TYPING_SUMMARY: str = "typing-summary"
    VCF_COVID: str = "vcf-covid"

    @classmethod
    def name(cls) -> str:
        return "Mutant Tags"

    @property
    def description(self) -> str:
        descriptions: dict[MutantTags, str] = {
            self.ARTIC_JSON: "GMS-Artic json file",
            self.ARTIC_QC: "GMS-Artic QC file",
            self.ARTIC_SUM: "GMS-Artic summary file",
            self.ARTIC_TYPE: "GMS-Artic typing file",
            self.ARTIC_VAR: "GMS-Artic variant file",
            self.CONSENSUS: "Consensus sequence",
            self.CONSENSUS_SAMPLE: "Separate sample consensus fasta",
            self.FOHM_DELIVERY: "Relevant for FoHM",
            self.INSTRUMENT_PROPERTIES: "Sequencing metadata",
            self.KOMPLETTERING: "Filetype specific for national uploading of microbial data",
            self.KS_AUX_RESULTS: "KS modification of pangolin called variants",
            self.KS_DELIVERY: "Relevant for Karolinska hospital/microbial routine",
            self.KS_RESULTS: "KS modification of pangolin typing results",
            self.MUTANT_CONFIG: "Config settings for mutant analysis",
            self.MUTANT_LOG: "SLURM log for mutant analysis",
            self.PANGOLIN: "Pangolin specific output",
            self.PANGOLIN_TYPING: "Pangolin typing results",
            self.PANGOLIN_TYPING_FOHM: "Pangolin typing report with only qc_pass samples included",
            self.TYPING_REPORT: "Results from typing",
            self.TYPING_SUMMARY: "Summary of results from typing",
            self.VCF_COVID: "VCF file containing covid data. Distinction req. by production",
        }
        return descriptions.get(self, "Description not available")


class RarediseaseTags(StrEnum):
    ANNOTATION: str = "annotation"
    EKLIPSE_DEL: str = "eklipse-del"
    EKLIPSE_GENES: str = "eklipse-gene"
    EKLIPSE_PNG: str = "eklipse-png"
    HAPLOGREP: str = "haplogrep"
    NGSBITS: str = "ngsbits-samplegender"
    SVDBQUERY: str = "svdbquery"
    SVDBQUERY_INDEX: str = "svdbquery-index"

    @classmethod
    def name(cls) -> str:
        return "Raredisease Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, RarediseaseTags] = {
            self.ANNOTATION: "Annotation tag",
            self.EKLIPSE_DEL: "Mitochondrial eKLIPse deletions",
            self.EKLIPSE_GENES: "Mitochondrial eKLIPse genes",
            self.EKLIPSE_PNG: "Mitochondrial eKLIPse png",
            self.HAPLOGREP: "Haplogrep",
            self.NGSBITS: "Result from SampleGender tool to determine sample sex",
            self.SVDBQUERY: "Query of SVDB results",
            self.SVDBQUERY_INDEX: "Query of SVDB results, index",
        }
        return descriptions.get(self, "Description not available")


class RnafusionTags(StrEnum):
    ARRIBA_VISUALISATION: str = "arriba-visualisation"
    FUSIONINSPECTOR_HTML: str = "fusioninspector-html"
    FUSIONREPORT: str = "fusionreport"

    @classmethod
    def name(cls) -> str:
        return "Rnafusion Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, RnafusionTags] = {
            self.ARRIBA_VISUALISATION: "Arriba visualization",
            self.FUSIONINSPECTOR_HTML: "Fusioninspector report",
            self.FUSIONREPORT: "Fusion-report analysis",
        }
        return descriptions.get(self, "Description not available")


class TaxprofilerTags(StrEnum):
    BRACKEN: str = "bracken"
    CENTRIFUGE: str = "centrifuge"
    COMBINED_REPORT: str = "combined-report"
    KRAKEN2: str = "kraken2"
    KRONA: str = "krona"
    METAGENOMICS_REPORT: str = "metagenomics-report"

    @classmethod
    def name(cls) -> str:
        return "Taxprofiler Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, TaxprofilerTags] = {
            self.BRACKEN: "Tool to compute the abundance of species and is companion program to "
            "kraken2",
            self.CENTRIFUGE: "Taxonomic sequence classifier for metagenomic sequences",
            self.COMBINED_REPORT: "A combined report of taxonomic classification from all samples",
            self.KRAKEN2: "Taxonomic sequence classifier that assigns taxonomic labels to "
            "metagenomic DNA sequences",
            self.KRONA: "Visualisation tool that allows to explore relative abundances and "
            "confidences within metagenomic classifications",
            self.METAGENOMICS_REPORT: "Report describing taxonomic classification results",
        }
        return descriptions.get(self, "Description not available")


class TomteTags(StrEnum):
    FRASER: str = "fraser"
    OUTRIDER: str = "outrider"

    @classmethod
    def name(cls) -> str:
        return "Tomte Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, TomteTags] = {
            self.FRASER: "Aberrant splicing calculated with DROP",
            self.OUTRIDER: "Aberrant expression calculated with DROP",
        }
        return descriptions.get(self, "Description not available")


class NextflowTags(StrEnum):
    NEXTFLOW_CONFIG: str = "nextflow-config"
    NEXTFLOW_PARAMS: str = "nextflow-params"
    SAMPLESHEET: str = "nextflow-samplesheet"
    SAMPLESHEET_VALID: str = "samplesheet-valid"
    SOFTWARE_VERSIONS: str = "software-versions"
    MANIFEST: str = "manifest"

    @classmethod
    def name(cls) -> str:
        return "Nextflow Tags"

    @property
    def description(self) -> str:
        descriptions: dict[str, NextflowTags] = {
            self.NEXTFLOW_CONFIG: "Nextflow config for analysis",
            self.NEXTFLOW_PARAMS: "Nextflow parameters file for analysis",
            self.SAMPLESHEET: "Samplesheet for analysis",
            self.SAMPLESHEET_VALID: "Validated samplesheet",
            self.SOFTWARE_VERSIONS: "List of all software used and their versions",
        }
        return descriptions.get(self, "Description not available")


class UsageTags(StrEnum):
    AUDIT: str = "audit"
    CG: str = "cg"
    CHANJO: str = "chanjo"
    CLINICAL_DELIVERY: str = "clinical-delivery"
    DELIVER: str = "deliver"  # Deprecated
    GENOTYPE: str = "genotype"
    JANUS: str = "janus"
    LONG_TERM_STORAGE: str = "long-term-storage"
    NIPT: str = "nipt"
    SCOUT: str = "scout"
    STORAGE: str = "storage"  # Deprecated


USAGE_TAGS: set[str] = {tag.value for tag in UsageTags}

COMMON_TAG_CATEGORIES: list[Any] = [
    AlignmentTags,
    AnalysisTags,
    BioinfoToolsTags,
    FamilyTags,
    QCTags,
    RawDataTags,
    ReportTags,
    VariantTags,
]


TAG_CATEGORIES: list[Any] = COMMON_TAG_CATEGORIES + [
    BalsamicTags,
    MicrosaltTags,
    MipTags,
    MutantTags,
    NextflowTags,
    RarediseaseTags,
    RnafusionTags,
    TaxprofilerTags,
    TomteTags,
]

ALL_TAGS: set[str] = {tag.value for category in TAG_CATEGORIES for tag in category}



"""Tags that are defined in Taxprofiler deliverables mapped to tags stored in Housekeeper."""

from cg_hermes.config.nextflow import NEXTFLOW_TAGS
from cg_hermes.constants.tags import (
    TaxprofilerTags,
    AnalysisTags,
    ReportTags,
    QCTags,
    BioinfoToolsTags,
    UsageTags,
)

TAXPROFILER_COMMON_TAGS = {
    frozenset({"kraken2_combined_report", "kraken2"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.KRAKEN2, TaxprofilerTags.COMBINED_REPORT],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"kraken2_report", "kraken2"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.KRAKEN2, TaxprofilerTags.METAGENOMICS_REPORT],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"krona_kraken_plot", "krona"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.KRAKEN2, TaxprofilerTags.KRONA, AnalysisTags.VISUALIZATION],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"krona_centrifuge_plot", "krona"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.KRAKEN2, TaxprofilerTags.KRONA, AnalysisTags.VISUALIZATION],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"bracken_combined_report", "bracken"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.BRACKEN, TaxprofilerTags.COMBINED_REPORT],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"bracken_report", "bracken"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.BRACKEN, TaxprofilerTags.METAGENOMICS_REPORT],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"centrifuge_combined_report", "centrifuge"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.CENTRIFUGE, TaxprofilerTags.COMBINED_REPORT],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"centrifuge_report", "centrifuge"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [TaxprofilerTags.CENTRIFUGE, TaxprofilerTags.METAGENOMICS_REPORT],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"multiqc-html", "report"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [ReportTags.MULTIQC_HTML],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc-json"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_JSON],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc-json", "multiqc-general-stats"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.GENERAL_STATS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-json", "multiqc-fastp"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, BioinfoToolsTags.FASTP],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-json", "multiqc-samtools-stats"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.SAMTOOLS_STATS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-json", "multiqc-kraken"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, TaxprofilerTags.KRAKEN2],
        "used_by": [UsageTags.JANUS],
    },
}

TAXPROFILER_TAGS = {**TAXPROFILER_COMMON_TAGS, **NEXTFLOW_TAGS}



"""Tags that are used in the fluffy workflow."""

FLUFFY_COMMON_TAGS = {
    frozenset(["multiqc"]): {
        "tags": ["multiqc-html"],
        "is_mandatory": True,
        "used_by": ["nipt"],
    },
    frozenset(["nipt_csv"]): {
        "tags": ["metrics"],
        "bundle_id": True,
        "is_mandatory": True,
        "used_by": ["nipt"],
    },
    frozenset(["nipt_first_pass_csv"]): {
        "tags": ["first-pass"],
        "bundle_id": True,
        "is_mandatory": False,
        "used_by": ["nipt"],
    },
    frozenset(["nipt_second_pass_csv"]): {
        "tags": ["second-pass"],
        "bundle_id": True,
        "is_mandatory": False,
        "used_by": ["nipt"],
    },
    frozenset(["wisecondor_aberrations"]): {
        "tags": ["wisecondor", "cnv"],
        "is_mandatory": True,
        "used_by": ["nipt"],
    },
}



"""Tags that are defined in rnafusion deliverables mapped to tags used in CG

The tag sets that exists in all files are set to mandatory. Tag sets that exists in < 4 deliverables are not
mandatory by default. However the tags that are available to a particular analysis is mandatory for that analysis.
"""

from cg_hermes.config.nextflow import NEXTFLOW_TAGS
from cg_hermes.constants.tags import (
    AlignmentTags,
    AnalysisTags,
    BioinfoToolsTags,
    NextflowTags,
    QCTags,
    RawDataTags,
    ReportTags,
    RnafusionTags,
    UsageTags,
)

RNAFUSION_COMMON_TAGS = {
    frozenset({"arriba"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [BioinfoToolsTags.ARRIBA, AnalysisTags.FUSION],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"arriba-visualisation", "arriba"}): {
        "is_mandatory": False,
        "bundle_id": True,
        "tags": [
            RnafusionTags.ARRIBA_VISUALISATION,
            AnalysisTags.VISUALIZATION,
            BioinfoToolsTags.ARRIBA,
            AnalysisTags.RESEARCH,
        ],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fusioncatcher"}): {
        "is_mandatory": True,
        "tags": [BioinfoToolsTags.FUSIONCATCHER, AnalysisTags.FUSION],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fusioncatcher-summary", "fusioncatcher"}): {
        "is_mandatory": True,
        "tags": [BioinfoToolsTags.FUSIONCATCHER_SUMMARY],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"star-fusion"}): {
        "is_mandatory": True,
        "tags": [BioinfoToolsTags.STAR_FUSION, AnalysisTags.FUSION],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fusionreport", "report"}): {
        "is_mandatory": True,
        "tags": [RnafusionTags.FUSIONREPORT, AnalysisTags.RESEARCH],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fusioninspector", "report"}): {
        "is_mandatory": False,
        "tags": [BioinfoToolsTags.FUSIONINSPECTOR],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fusioninspector-html", "report"}): {
        "is_mandatory": False,
        "tags": [RnafusionTags.FUSIONINSPECTOR_HTML, AnalysisTags.RESEARCH],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc-html", "report"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_HTML, RawDataTags.RNA],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"star-fusion-cram", "star-fusion"}): {
        "is_mandatory": True,
        "tags": [AlignmentTags.CRAM],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"star-fusion-cram-index", "star-fusion"}): {
        "is_mandatory": True,
        "tags": [AlignmentTags.CRAM_INDEX],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"star-align-gene-counts", "star-align"}): {
        "is_mandatory": True,
        "tags": [ReportTags.GENE_COUNTS],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc-json"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_JSON],
        "used_by": [UsageTags.CG],
    },
    frozenset({"vcf-fusion", "vcf-collect"}): {
        "is_mandatory": False,
        "tags": [AnalysisTags.VCF_FUSION],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc-fastp", "multiqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, BioinfoToolsTags.FASTP],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-picard-duplicates", "multiqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_DUPLICATES],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-picard-insert-size", "multiqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_INSERT_SIZE],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-picard-rnaseq", "multiqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_RNASEQ],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-general-stats", "multiqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.GENERAL_STATS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc-star", "multiqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.STAR],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"samplesheet-valid"}): {
        "is_mandatory": True,
        "bundle_id": True,
        "tags": [NextflowTags.SAMPLESHEET_VALID],
        "used_by": [UsageTags.CG],
    },
}

RNAFUSION_TAGS = {**RNAFUSION_COMMON_TAGS, **NEXTFLOW_TAGS}



"""
cg_hermes.__main__

The main entry point for the command line interface.

Invoke as ``hermes`` (if installed)
or ``python -m cg_hermes`` (no install required).
"""

import sys

from cg_hermes.cli.base import app


def main():
    sys.exit(app())


if __name__ == "__main__":
    main()



"""Tags that are defined in Balsamic deliverables mapped to tags used in CG

The tag sets that exists in all files are set to mandatory. Tag sets that exists in < 4 deliverables are not
mandatory by default. However the tags that are available to a particular analysis is mandatory for that analysis.
"""

RAW_TAGS = {
    # Config, QC and reports (PANEL & WGS)
    "config.json": ["balsamic-config"],
    "report.html": ["balsamic-report"],
    "BALSAMIC_X.X.X_graph.pdf": ["balsamic-dag"],
    "multiqc_report.html": ["html", "multiqc-html"],
    "multiqc_data.json": ["json", "multiqc-json"],
    # Custom QC
    "metrics_deliverables.yaml": ["yaml", "qc-metrics-yaml"],
    # Alignment files (PANEL & WGS)
    "tumor.cram": ["cram", "tumor-cram"],
    "tumor.cram.crai": ["cram", "tumor-cram-index"],
    "normal.cram": ["cram", "normal-cram"],
    "normal.cram.crai": ["cram", "normal-cram-index"],
    # UMI alignment files (PANEL)
    "tumor_umi_consensusfiltered.cram": ["cram", "umi-tumor-cram"],
    "tumor_umi_consensusfiltered.cram.crai": ["cram", "umi-tumor-cram-index"],
    "normal_umi_consensusfiltered.cram": ["cram", "umi-normal-cram"],
    "normal_umi_consensusfiltered.cram.crai": ["cram", "umi-normal-cram-index"],
    # Germline SNVs (PANEL & WGS)
    "germline.tumor.dnascope.vcf.gz": [
        "vcf-tumor",
        "snv",
        "dnascope",
        "germline-vcf-tumor",
    ],
    "germline.tumor.dnascope.vcf.gz.tbi": [
        "vcf-tumor",
        "snv",
        "dnascope",
        "germline-vcf-tumor-index",
    ],
    "germline.normal.dnascope.vcf.gz": [
        "vcf-normal",
        "snv",
        "dnascope",
        "germline-vcf-normal",
    ],
    "germline.normal.dnascope.vcf.gz.tbi": [
        "vcf-normal",
        "snv",
        "dnascope",
        "germline-vcf-normal-index",
    ],
    "genotype.normal.dnascope.vcf.gz": [
        "vcf-dnascope",
        "genotype-vcf-dnascope",
    ],
    "genotype.normal.dnascope.vcf.gz.tbi": [
        "vcf-dnascope",
        "genotype-vcf-dnascope-index",
    ],
    # Germline SVs (PANEL & WGS)
    "germline.tumor.manta_germline.vcf.gz": [
        "vcf-tumor",
        "sv",
        "manta-germline",
        "germline-vcf-tumor",
    ],
    "germline.tumor.manta_germline.vcf.gz.tbi": [
        "vcf-tumor",
        "sv",
        "manta-germline",
        "germline-vcf-tumor-index",
    ],
    "germline.normal.manta_germline.vcf.gz": [
        "vcf-normal",
        "sv",
        "manta-germline",
        "germline-vcf-normal",
    ],
    "germline.normal.manta_germline.vcf.gz.tbi": [
        "vcf-normal",
        "sv",
        "manta-germline",
        "germline-vcf-normal-index",
    ],
    # Merged SV (manta, delly) and CNV (cnvkit, ascat) callers (PANEL & WGS)
    "svdb.vcf.gz": ["vcf-svdb", "research-vcf-svdb"],
    "svdb.vcf.gz.tbi": ["vcf-svdb", "research-vcf-svdb-index"],
    "svdb.research.filtered.pass.vcf.gz": ["vcf-pass-svdb", "research-vcf-pass-svdb"],
    "svdb.research.filtered.pass.vcf.gz.tbi": ["vcf-pass-svdb", "research-vcf-pass-svdb-index"],
    "svdb.clinical.filtered.pass.vcf.gz": ["vcf-pass-svdb", "clinical-vcf-pass-svdb"],
    "svdb.clinical.filtered.pass.vcf.gz.tbi": ["vcf-pass-svdb", "clinical-vcf-pass-svdb-index"],
    # SNVs/InDels
    "vardict.vcf.gz": [
        "vcf-vardict",
        "research-vcf-vardict",
    ],
    "vardict.vcf.gz.tbi": [
        "vcf-vardict",
        "research-vcf-vardict-index",
    ],
    "tnscope.vcf.gz": [
        "vcf-tnscope",
        "research-vcf-tnscope",
    ],
    "tnscope.vcf.gz.tbi": [
        "vcf-tnscope",
        "research-vcf-tnscope-index",
    ],
    "tnscope.research.vcf.gz": [
        "vcf-snv-unfiltered",
        "tnscope",
        "research-vcf-snv-unfiltered",
    ],
    "tnscope.research.vcf.gz.tbi": [
        "vcf-snv-unfiltered",
        "tnscope",
        "research-vcf-snv-unfiltered-index",
    ],
    "tnscope.clinical.filtered.pass.vcf.gz": [
        "vcf-pass-tnscope",
        "snv",
        "clinical-vcf-pass-tnscope",
    ],
    "tnscope.clinical.filtered.pass.vcf.gz.tbi": [
        "vcf-pass-tnscope",
        "snv",
        "clinical-vcf-pass-tnscope-index",
    ],
    "tnscope.clinical.scored.vcf.gz": [
        "tnscope",
        "scored-vcf-clinical",
        "snv",
        "vcf-clinical",
    ],
    "tnscope.clinical.scored.vcf.gz.tbi": [
        "tnscope",
        "scored-vcf-clinical-index",
        "snv",
        "vcf-clinical",
    ],
    "merged.research.vcf.gz": [
        "vcf-snv-unfiltered",
        "merged",
        "research-vcf-snv-unfiltered",
    ],
    "merged.research.vcf.gz.tbi": [
        "vcf-snv-unfiltered",
        "merged",
        "research-vcf-snv-unfiltered-index",
    ],
    "merged.clinical.filtered.pass.vcf.gz": [
        "vcf-pass-merged",
        "snv",
        "clinical-vcf-pass-merged",
    ],
    "merged.clinical.filtered.pass.vcf.gz.tbi": [
        "vcf-pass-merged",
        "snv",
        "clinical-vcf-pass-merged-index",
    ],
    "merged.clinical.scored.vcf.gz": [
        "merged",
        "scored-vcf-clinical",
        "snv",
        "vcf-clinical",
    ],
    "merged.clinical.scored.vcf.gz.tbi": [
        "merged",
        "scored-vcf-clinical-index",
        "snv",
        "vcf-clinical",
    ],
    # UMI SNVs/INDELs (PANEL)
    "tnscope_umi.vcf.gz": [
        "vcf-tnscope-umi",
        "research-vcf-tnscope-umi",
    ],
    "tnscope_umi.vcf.gz.tbi": [
        "vcf-tnscope-umi",
        "research-vcf-tnscope-umi-index",
    ],
    "tnscope_umi.research.vcf.gz": [
        "research-vcf-snv-unfiltered",
        "tnscope-umi",
        "vcf-snv-unfiltered",
    ],
    "tnscope_umi.research.vcf.gz.tbi": [
        "research-vcf-snv-unfiltered-index",
        "tnscope-umi",
        "vcf-snv-unfiltered",
    ],
    "tnscope_umi.clinical.filtered.pass.vcf.gz": [
        "vcf-pass-tnscope-umi",
        "snv",
        "clinical-vcf-pass-tnscope-umi",
    ],
    "tnscope_umi.clinical.filtered.pass.vcf.gz.tbi": [
        "vcf-pass-tnscope-umi",
        "snv",
        "clinical-vcf-pass-tnscope-umi-index",
    ],
    "tnscope_umi.clinical.scored.vcf.gz": [
        "scored-vcf-clinical",
        "tnscope-umi",
        "snv",
        "vcf-clinical",
    ],
    "tnscope_umi.clinical.scored.vcf.gz.tbi": [
        "scored-vcf-clinical-index",
        "tnscope-umi",
        "snv",
        "vcf-clinical",
    ],
    # CNVs (PANEL)
    "tumor.merged.cns": ["cns", "cnv-cns"],
    "tumor.merged.cnr": ["cnr", "cnv-cnr"],
    "gene_metrics": ["gene-metrics", "cnv-gene-metrics"],
    "cnvkit.vcf2cytosure.cgh": ["cgh-tumor", "cnv-somatic-cgh-tumor"],
    # CNVs (WGS)
    "ascat.copynumber.txt.gz": ["ascat-copynumber", "clinical-ascat-copynumber"],
    "dellycnv.cov.gz": ["rd-delly", "clinical-rd-delly"],
    "tumor.vcf2cytosure.cgh": ["cgh-tumor", "cnv-somatic-cgh-tumor"],
    "normal.vcf2cytosure.cgh": ["cgh-normal", "cnv-somatic-cgh-normal"],
    "cov.bed.gz": ["cov", "gens-bed", "cnv-gens-bed"],
    "cov.bed.gz.tbi": ["cov", "gens-bed", "cnv-gens-bed-index"],
    "baf.bed.gz": ["baf", "gens-bed", "cnv-gens-bed"],
    "baf.bed.gz.tbi": ["baf", "gens-bed", "cnv-gens-bed-index"],
    # CNV report
    "report.pdf": ["cnv-report-pdf", "clinical-cnv-report-pdf"],
    "cnvpytor.circular.png": ["circular-cnvpytor", "clinical-circular-cnvpytor"],
    "cnvpytor.scatter.png": ["scatter-cnvpytor", "clinical-scatter-cnvpytor"],
    "ascat.ascatprofile.png": ["plot-ascat-profile", "clinical-plot-ascat-profile"],
    "ascat.rawprofile.png": ["plot-raw-profile", "clinical-plot-raw-profile"],
    "ascat.ASPCF.png": ["plot-aspcf", "clinical-plot-aspcf"],
    "ascat.tumor.png": ["plot-tumor", "clinical-plot-tumor"],
    "ascat.germline.png": ["plot-germline", "clinical-plot-germline"],
    "ascat.sunrise.png": ["plot-sunrise", "clinical-plot-sunrise"],
    # SVs (WGS)
    "tumor.tiddit_cov.bed": ["cov-tumor-tiddit", "clinical-cov-tumor-tiddit"],
    "normal.tiddit_cov.bed": ["cov-normal-tiddit", "clinical-cov-normal-tiddit"],
    # TMB (PANEL)
    "tnscope_umi.balsamic_stat": ["snv", "tnscope-umi", "tmb", "research-tmb"],
    "merged.balsamic_stat": ["snv", "merged", "tmb", "research-tmb"],
    # TMB (WGS)
    "tnscope.balsamic_stat": ["snv", "tnscope", "tmb", "research-tmb"],
    # MSI (PANEL and WGS)
    "msisensorpro.msi": ["msi-result", "research-msi-result"],
}

QC_TAGS = {
    # Config, QC and reports (PANEL & WGS)
    frozenset(RAW_TAGS["config.json"]): {  # BALSAMIC config json
        "tags": ["balsamic-config"],
        "is_mandatory": True,
        "used_by": ["audit", "cg"],
    },
    frozenset(RAW_TAGS["report.html"]): {  # BALSAMIC report html
        "tags": ["balsamic-report"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(RAW_TAGS["BALSAMIC_X.X.X_graph.pdf"]): {  # DAG
        "tags": ["balsamic-dag"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    frozenset(RAW_TAGS["multiqc_report.html"]): {  # MultiQC html
        "tags": ["multiqc-html"],
        "is_mandatory": True,
        "bundle_id": True,
        "used_by": ["audit", "deliver", "scout"],
    },
    frozenset(RAW_TAGS["multiqc_data.json"]): {  # MultiQC json
        "tags": ["multiqc-json"],
        "is_mandatory": True,
        "used_by": ["audit"],
    },
    # Custom QC
    frozenset(RAW_TAGS["metrics_deliverables.yaml"]): {  # QC metrics
        "tags": ["qc-metrics", "deliverable"],
        "is_mandatory": True,
        "used_by": ["audit", "cg"],
    },
}

ALIGNMENT_TAGS = {
    # Alignment files (PANEL & WGS)
    frozenset(RAW_TAGS["tumor.cram"]): {  # cram (tumor)
        "tags": ["tumor", "cram"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["tumor.cram.crai"]): {
        "tags": ["tumor", "cram-index"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["normal.cram"]): {  # cram (normal)
        "tags": ["normal", "cram"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["normal.cram.crai"]): {
        "tags": ["normal", "cram-index"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
}

GERMLINE_TAGS = {
    # Germline SNVs (PANEL & WGS)
    frozenset(RAW_TAGS["germline.tumor.dnascope.vcf.gz"]): {
        "tags": ["germline", "vcf-snv-germline-tumor"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["germline.tumor.dnascope.vcf.gz.tbi"]): {
        "tags": ["germline", "vcf-snv-germline-tumor-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz"]): {
        "tags": ["germline", "vcf-snv-germline-normal"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz.tbi"]): {
        "tags": ["germline", "vcf-snv-germline-normal-index"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz"]): {
        "tags": ["dnascope", "normal", "vcf"],
        "is_mandatory": False,
        "used_by": ["cg", "genotype"],
    },
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz.tbi"]): {
        "tags": ["dnascope", "normal", "vcf-index"],
        "is_mandatory": False,
        "used_by": ["cg", "genotype"],
    },
    # Germline SVs (PANEL & WGS)
    frozenset(RAW_TAGS["germline.tumor.manta_germline.vcf.gz"]): {
        "tags": ["germline", "vcf-sv-germline-tumor"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["germline.tumor.manta_germline.vcf.gz.tbi"]): {
        "tags": ["germline", "vcf-sv-germline-tumor-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz"]): {
        "tags": ["germline", "vcf-sv-germline-normal"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz.tbi"]): {
        "tags": ["germline", "vcf-sv-germline-normal-index"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
}

CALLERS_TAGS = {
    # Merged SV (manta, delly) and CNV (cnvkit, ascat) callers (PANEL & WGS)
    frozenset(RAW_TAGS["svdb.vcf.gz"]): {
        "tags": ["svdb", "vcf-sv"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["svdb.vcf.gz.tbi"]): {
        "tags": ["svdb", "vcf-sv-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["svdb.research.filtered.pass.vcf.gz"]): {
        "tags": ["svdb", "vcf-sv-research"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["svdb.research.filtered.pass.vcf.gz.tbi"]): {
        "tags": ["svdb", "vcf-sv-research-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["svdb.clinical.filtered.pass.vcf.gz"]): {
        "tags": ["svdb", "vcf-sv-clinical"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["svdb.clinical.filtered.pass.vcf.gz.tbi"]): {
        "tags": ["svdb", "vcf-sv-clinical-index"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    # SNVs (WGS & Panel)
    frozenset(RAW_TAGS["tnscope.vcf.gz"]): {
        "tags": ["tnscope", "vcf-snv"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope.vcf.gz.tbi"]): {
        "tags": ["tnscope", "vcf-snv-index"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    # SNVs (WGS)
    frozenset(RAW_TAGS["tnscope.research.vcf.gz"]): {
        "tags": ["tnscope", "vcf-snv-research-unfiltered"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope.research.vcf.gz.tbi"]): {
        "tags": ["tnscope", "vcf-snv-research-unfiltered-index"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tnscope.clinical.filtered.pass.vcf.gz"]): {
        "tags": ["tnscope", "vcf-snv-clinical"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["tnscope.clinical.filtered.pass.vcf.gz.tbi"]): {
        "tags": ["tnscope", "vcf-snv-clinical-index"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["tnscope.clinical.scored.vcf.gz"]): {
        "tags": ["tnscope", "vcf-snv-clinical-scored"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["tnscope.clinical.scored.vcf.gz.tbi"]): {
        "tags": ["tnscope", "vcf-snv-clinical-scored-index"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    # CNVs (WGS)
    frozenset(RAW_TAGS["ascat.copynumber.txt.gz"]): {
        "tags": ["ascatngs", "metrics"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["dellycnv.cov.gz"]): {
        "tags": ["delly", "coverage"],
        "is_mandatory": True,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tumor.vcf2cytosure.cgh"]): {
        "tags": ["tiddit", "tumor", "vcf2cytosure"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["normal.vcf2cytosure.cgh"]): {
        "tags": ["tiddit", "normal", "vcf2cytosure"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    # SNVs/INDELs (PANEL)
    frozenset(RAW_TAGS["vardict.vcf.gz"]): {
        "tags": ["vardict", "vcf-snv"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["vardict.vcf.gz.tbi"]): {
        "tags": ["vardict", "vcf-snv-index"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["merged.research.vcf.gz"]): {
        "tags": ["merged", "vcf-snv-research-unfiltered"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["merged.research.vcf.gz.tbi"]): {
        "tags": ["merged", "vcf-snv-research-unfiltered-index"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["merged.clinical.filtered.pass.vcf.gz"]): {
        "tags": ["merged", "vcf-snv-clinical"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["merged.clinical.filtered.pass.vcf.gz.tbi"]): {
        "tags": ["merged", "vcf-snv-clinical-index"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["merged.clinical.scored.vcf.gz"]): {
        "tags": ["merged", "vcf-snv-clinical-scored"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["merged.clinical.scored.vcf.gz.tbi"]): {
        "tags": ["merged", "vcf-snv-clinical-scored-index"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    # CNVs (PANEL)
    frozenset(RAW_TAGS["tumor.merged.cns"]): {
        "tags": ["cnvkit", "metrics", "segments"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["tumor.merged.cnr"]): {
        "tags": ["cnvkit", "regions"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["gene_metrics"]): {
        "tags": ["cnvkit", "metrics", "genes"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["cnvkit.vcf2cytosure.cgh"]): {
        "tags": ["cnvkit", "tumor", "vcf2cytosure"],
        "is_mandatory": False,
        "used_by": ["deliver", "scout"],
    },
    # CNVs visualization in GENS
    frozenset(RAW_TAGS["cov.bed.gz"]): {
        "tags": ["gens", "coverage", "bed"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    frozenset(RAW_TAGS["cov.bed.gz.tbi"]): {
        "tags": ["gens", "coverage", "bed-index"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    frozenset(RAW_TAGS["baf.bed.gz"]): {
        "tags": ["gens", "fracsnp", "bed"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    frozenset(RAW_TAGS["baf.bed.gz.tbi"]): {
        "tags": ["gens", "fracsnp", "bed-index"],
        "is_mandatory": True,
        "used_by": ["scout"],
    },
    # CNV report
    frozenset(RAW_TAGS["report.pdf"]): {
        "tags": ["cnv-report"],
        "is_mandatory": True,
        "used_by": ["deliver", "scout"],
    },
    frozenset(RAW_TAGS["cnvpytor.circular.png"]): {
        "tags": ["circular-plot", "cnvpytor", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["cnvpytor.scatter.png"]): {
        "tags": ["scatter-plot", "cnvpytor", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["ascat.ascatprofile.png"]): {
        "tags": ["profile-plot", "ascatngs", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["ascat.rawprofile.png"]): {
        "tags": ["raw-profile-plot", "ascatngs", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["ascat.ASPCF.png"]): {
        "tags": ["aspcf-plot", "ascatngs", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["ascat.tumor.png"]): {
        "tags": ["tumor-plot", "ascatngs", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["ascat.germline.png"]): {
        "tags": ["germline-plot", "ascatngs", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    frozenset(RAW_TAGS["ascat.sunrise.png"]): {
        "tags": ["sunrise-plot", "ascatngs", "visualization"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    # SVs (WGS)
    frozenset(RAW_TAGS["tumor.tiddit_cov.bed"]): {
        "tags": ["tiddit", "tumor", "coverage"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    frozenset(RAW_TAGS["normal.tiddit_cov.bed"]): {
        "tags": ["tiddit", "normal", "coverage"],
        "is_mandatory": False,
        "used_by": ["deliver"],
    },
    # TMB (WGS)
    frozenset(RAW_TAGS["tnscope.balsamic_stat"]): {
        "tags": ["research", "tnscope", "tmb"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    # TMB (TGA)
    frozenset(RAW_TAGS["merged.balsamic_stat"]): {
        "tags": ["research", "merged", "tmb"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
    # MSI (WGS and TGA)
    frozenset(RAW_TAGS["msisensorpro.msi"]): {
        "tags": ["research", "msi"],
        "is_mandatory": False,
        "used_by": ["storage"],
    },
}

BALSAMIC_TAGS = {
    **QC_TAGS,
    **ALIGNMENT_TAGS,
    **GERMLINE_TAGS,
    **CALLERS_TAGS,
}

TUMOR_ONLY_WGS_TAGS = {
    # QC
    # SNVs (WGS)
    frozenset(RAW_TAGS["tnscope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.research.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.research.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.scored.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.scored.vcf.gz.tbi"]): {"is_mandatory": True},
    # CNVs (WGS)
    frozenset(RAW_TAGS["tumor.vcf2cytosure.cgh"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["cnvpytor.circular.png"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["cnvpytor.scatter.png"]): {"is_mandatory": True},
    # SVs (WGS)
    frozenset(RAW_TAGS["tumor.tiddit_cov.bed"]): {"is_mandatory": True},
    # TMB (WGS)
    frozenset(RAW_TAGS["tnscope.balsamic_stat"]): {"is_mandatory": True},
}

TUMOR_NORMAL_WGS_TAGS = {
    # QC
    # Alignment files (PANEL & WGS)
    frozenset(RAW_TAGS["normal.cram"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal.cram.crai"]): {"is_mandatory": True},
    # Germline SNVs (PANEL & WGS)
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz.tbi"]): {"is_mandatory": True},
    # SNVs (WGS)
    frozenset(RAW_TAGS["tnscope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.research.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.research.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.scored.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.clinical.scored.vcf.gz.tbi"]): {"is_mandatory": True},
    # CNVs (WGS)
    frozenset(RAW_TAGS["ascat.copynumber.txt.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tumor.vcf2cytosure.cgh"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal.vcf2cytosure.cgh"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["ascat.ascatprofile.png"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["ascat.rawprofile.png"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["ascat.ASPCF.png"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["ascat.tumor.png"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["ascat.germline.png"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["ascat.sunrise.png"]): {"is_mandatory": True},
    # SVs (WGS)
    frozenset(RAW_TAGS["tumor.tiddit_cov.bed"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal.tiddit_cov.bed"]): {"is_mandatory": True},
    # MSI (WGS)
    frozenset(RAW_TAGS["msisensorpro.msi"]): {"is_mandatory": True},
}

TUMOR_ONLY_PANEL_TAGS = {
    # SNVs/INDELs (PANEL)
    frozenset(RAW_TAGS["vardict.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.research.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.research.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.scored.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.scored.vcf.gz.tbi"]): {"is_mandatory": True},
    # CNVs (PANEL)
    frozenset(RAW_TAGS["tumor.merged.cns"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tumor.merged.cnr"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["gene_metrics"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["cnvkit.vcf2cytosure.cgh"]): {"is_mandatory": True},
    # TMB (PANEL)
    frozenset(RAW_TAGS["merged.balsamic_stat"]): {"is_mandatory": True},
}

TUMOR_NORMAL_PANEL_TAGS = {
    # QC
    # Alignment files (PANEL & WGS)
    frozenset(RAW_TAGS["normal.cram"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["normal.cram.crai"]): {"is_mandatory": True},
    # Germline SNVs (PANEL & WGS)
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.dnascope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["germline.normal.manta_germline.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["genotype.normal.dnascope.vcf.gz.tbi"]): {"is_mandatory": True},
    # SNVs/INDELs (PANEL)
    frozenset(RAW_TAGS["vardict.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["vardict.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tnscope.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.research.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.research.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.filtered.pass.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.filtered.pass.vcf.gz.tbi"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.scored.vcf.gz"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["merged.clinical.scored.vcf.gz.tbi"]): {"is_mandatory": True},
    # CNVs (PANEL)
    frozenset(RAW_TAGS["tumor.merged.cns"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["tumor.merged.cnr"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["gene_metrics"]): {"is_mandatory": True},
    frozenset(RAW_TAGS["cnvkit.vcf2cytosure.cgh"]): {"is_mandatory": True},
    # TMB (PANEL)
    frozenset(RAW_TAGS["merged.balsamic_stat"]): {"is_mandatory": True},
    # MSI (PANEL)
    frozenset(RAW_TAGS["msisensorpro.msi"]): {"is_mandatory": True},
}



from pydantic import BaseModel, field_validator

from cg_hermes.constants.tags import ALL_TAGS, USAGE_TAGS


class TagMap(BaseModel):
    tags: list[str]
    is_mandatory: bool
    used_by: list[str]
    bundle_id: bool | None = False

    @field_validator("tags")
    @classmethod
    def check_tags(cls, tags: list[str]):
        for tag in tags:
            if tag not in ALL_TAGS:
                raise ValueError(tags, f"{tag} not a valid tag")
        return tags

    @field_validator("used_by")
    @classmethod
    def check_usage(cls, used_by: list[str]):
        for user in used_by:
            if user not in USAGE_TAGS:
                raise ValueError(used_by, f"{user} not a valid user")
        return used_by


class CGTag(BaseModel):
    path: str
    tags: list[str]
    mandatory: bool



from typing import FrozenSet

from pydantic import BaseModel, field_validator

from cg_hermes.models.tags import CGTag

# Classes to represent file entries in the deliverables files


class FileBase(BaseModel):
    """Definition for elements in deliverables file"""

    path: str
    tag: str | None = None
    id: str


class MipFile(FileBase):
    """Definition for elements in MIP deliverables"""

    format: str
    path_index: str | None = None
    step: str


class MicrosaltFile(FileBase):
    """Definition for elements in Microsalt deliverables"""

    format: str
    step: str


class MutantFile(FileBase):
    """Definition for elements in Mutant deliverables"""

    format: str
    step: str


class NfAnalysisFile(FileBase):
    """Definition for elements in deliverables for nextflow workflows."""

    format: str
    path_index: str | None = None
    step: str


class BalsamicFile(FileBase):
    """Definition of elements in Balsamic deliverables."""

    format: str | None = None
    tag: list[str]

    @field_validator("tag", mode="before")
    @classmethod
    def split_str(cls, tag: list[str]):
        return tag.split(",") if isinstance(tag, str) else tag


# Classes to represent deliverable files


class WorkflowDeliverables(BaseModel):
    """Specification for a general deliverables file"""

    files: list[FileBase]


class MipDeliverables(WorkflowDeliverables):
    """Specification for a MIP specific deliverables file"""

    files: list[MipFile]


class MicrosaltDeliverables(WorkflowDeliverables):
    """Specification for a MIP specific deliverables file"""

    files: list[MicrosaltFile]


class BalsamicDeliverables(WorkflowDeliverables):
    """Specification for a BALSAMIC specific deliverables file"""

    files: list[BalsamicFile]


class FluffyDeliverables(WorkflowDeliverables):
    """Specification for a FLUFFY specific deliverables file"""

    files: list[FileBase]


class MutantDeliverables(WorkflowDeliverables):
    """Specification for a MUTANT specific deliverables file"""

    files: list[MutantFile]


class NfAnalysisDeliverables(WorkflowDeliverables):
    """Specification for a deliverable file for a nextflow workflow."""

    files: list[NfAnalysisFile]


class CGDeliverables(WorkflowDeliverables):
    """Class that specifies the output to CG"""

    workflow: str
    bundle_id: str
    files: list[CGTag]


class TagBase(BaseModel):
    """Internal mapping of deliverable file content"""

    tags: FrozenSet[str]
    subject_id: str
    path: str
    path_index: str | None = None



"""Tags that are defined in Tomte deliverables mapped to tags used in CG

The tag sets that exists in all files are set to mandatory. Tag sets that exists in < 4 deliverables are not
mandatory by default. However, the tags that are available to a particular analysis is mandatory for that analysis.
"""

from cg_hermes.config.nextflow import NEXTFLOW_TAGS
from cg_hermes.constants.tags import (
    AlignmentTags,
    UsageTags,
    QCTags,
    ReportTags,
    BioinfoToolsTags,
    RawDataTags,
    NextflowTags,
    AnalysisTags,
    TomteTags,
    VariantTags,
)

TOMTE_COMMON_TAGS = {
    frozenset({"transcript-counts", "salmon"}): {
        "is_mandatory": True,
        "tags": [BioinfoToolsTags.SALMON_QUANT],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"research-vcf", "snv"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.RESEARCH, VariantTags.VCF, VariantTags.SNV],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"research-vcf-index", "snv"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.RESEARCH, VariantTags.VCF_INDEX, VariantTags.SNV],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"clinical-vcf", "snv"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.CLINICAL, VariantTags.VCF, VariantTags.SNV],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"clinical-vcf-index", "snv"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.CLINICAL, VariantTags.VCF_INDEX, VariantTags.SNV],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fraser", "fraser-clinical"}): {
        "is_mandatory": False,
        "tags": [TomteTags.FRASER, AnalysisTags.CLINICAL],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"fraser", "fraser-research"}): {
        "is_mandatory": False,
        "tags": [TomteTags.FRASER, AnalysisTags.RESEARCH],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"outrider", "outrider-clinical"}): {
        "is_mandatory": False,
        "tags": [TomteTags.OUTRIDER, AnalysisTags.CLINICAL],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"outrider", "outrider-research"}): {
        "is_mandatory": False,
        "tags": [TomteTags.OUTRIDER, AnalysisTags.RESEARCH],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"star", "star-cram"}): {
        "is_mandatory": True,
        "tags": [AlignmentTags.CRAM],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.SCOUT],
    },
    frozenset({"star", "star-cram-index"}): {
        "is_mandatory": True,
        "tags": [AlignmentTags.CRAM_INDEX],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"coverage", "bigwig"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.COVERAGE, AnalysisTags.BIGWIG],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"junction", "bed"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.JUNCTION, AnalysisTags.BED],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"junction", "bed-index"}): {
        "is_mandatory": True,
        "tags": [AnalysisTags.JUNCTION, AnalysisTags.BED_INDEX],
        "used_by": [UsageTags.CLINICAL_DELIVERY],
    },
    frozenset({"stringtie", "assembly"}): {
        "is_mandatory": True,
        "tags": [BioinfoToolsTags.STRINGTIE, AnalysisTags.ASSEMBLY],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"gffcompare"}): {
        "is_mandatory": True,
        "tags": [BioinfoToolsTags.GFFCOMPARE],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"star", "raw-gene-counts"}): {
        "is_mandatory": True,
        "tags": [ReportTags.GENE_COUNTS],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"nextflow-config"}): {
        "is_mandatory": True,
        "tags": [NextflowTags.NEXTFLOW_CONFIG],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"nextflow-params"}): {
        "is_mandatory": True,
        "tags": [NextflowTags.NEXTFLOW_PARAMS],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc", "multiqc-html"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_HTML, RawDataTags.RNA],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.SCOUT, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc", "multiqc-json"}): {
        "is_mandatory": True,
        "tags": [ReportTags.MULTIQC_JSON],
        "used_by": [UsageTags.CLINICAL_DELIVERY, UsageTags.LONG_TERM_STORAGE],
    },
    frozenset({"multiqc", "multiqc-fastp"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, BioinfoToolsTags.FASTP],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-fastqc"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, BioinfoToolsTags.FASTQC],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-bcftools-stats"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.BCFTOOLS_STATS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-gffcompare"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, BioinfoToolsTags.GFFCOMPARE],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-rnaseq"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_RNASEQ],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-star"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.STAR],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-general-stats"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.GENERAL_STATS],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"multiqc", "multiqc-picard-insert-size"}): {
        "is_mandatory": True,
        "tags": [QCTags.QC_METRICS, ReportTags.MULTIQC, ReportTags.PICARD_INSERT_SIZE],
        "used_by": [UsageTags.JANUS],
    },
    frozenset({"samplesheet"}): {
        "is_mandatory": True,
        "tags": [NextflowTags.SAMPLESHEET],
        "used_by": [UsageTags.CG, UsageTags.LONG_TERM_STORAGE],
    },
}

TOMTE_TAGS = {**TOMTE_COMMON_TAGS, **NEXTFLOW_TAGS}



"""CLI for converting between formats."""

import json
import logging
from pathlib import Path

import typer
from pydantic import ValidationError

from cg_hermes.cli.common import get_deliverables
from cg_hermes.constants.workflow import CancerAnalysisType, Workflow
from cg_hermes.deliverables import Deliverables
from cg_hermes.exceptions import MissingFileError
from cg_hermes.models.workflow_deliverables import CGDeliverables
from cg_hermes.validate import get_deliverables_obj

LOG = logging.getLogger(__name__)

app = typer.Typer()


@app.command(name="deliverables")
def convert_cmd(
    deliverables_file: Path,
    workflow: Workflow = typer.Option(..., help="Specify the workflow"),
    analysis_type: CancerAnalysisType = typer.Option(None, help="Specify the analysis type"),
    force: bool = typer.Option(False, "--force", "-f", help="Bypass deliverables file validation"),
) -> None:
    """
    Convert deliverable file to CG format.

    Raises:
        typer.Abort: If an error occurs during conversion or validation.
    """
    LOG.info(f"Convert deliverable file: {deliverables_file} from workflow {workflow} to CG format")
    raw_deliverables: dict[str, list[dict[str, str]]] = get_deliverables(deliverables_file)
    try:
        deliverables: Deliverables = get_deliverables_obj(
            deliverables=raw_deliverables, workflow=workflow, analysis_type=analysis_type
        )
        deliverables.validate_mandatory_files(force)
    except SyntaxError as error:
        LOG.error(error)
        raise typer.Abort()
    except (ValidationError, MissingFileError) as error:
        LOG.error(error)
        LOG.error(f"File: {deliverables_file} does not follow the specification")
        raise typer.Abort()
    cg_deliverables: CGDeliverables = deliverables.convert_to_cg_deliverables()
    typer.echo(json.dumps(cg_deliverables.dict(), indent=2))



import copy
import logging
from enum import StrEnum

import typer
from tabulate import tabulate

from cg_hermes.config.balsamic_umi import BALSAMIC_UMI_TAGS
from cg_hermes.config.fluffy import FLUFFY_COMMON_TAGS
from cg_hermes.config.microsalt import MICROSALT_COMMON_TAGS
from cg_hermes.config.mip_dna import MIP_DNA_TAGS
from cg_hermes.config.mip_rna import MIP_RNA_TAGS
from cg_hermes.config.mutant import MUTANT_COMMON_TAGS
from cg_hermes.config.raredisease import RAREDISEASE_TAGS
from cg_hermes.config.rnafusion import RNAFUSION_TAGS
from cg_hermes.config.taxprofiler import TAXPROFILER_TAGS
from cg_hermes.config.tomte import TOMTE_TAGS
from cg_hermes.constants.tags import COMMON_TAG_CATEGORIES
from cg_hermes.constants.workflow import Workflow

app = typer.Typer()

LOG = logging.getLogger(__name__)


def get_table(tags: dict):
    table = []
    for workflow_tags, tag_info in tags.items():
        row = [
            ", ".join(workflow_tags),
            str(tag_info["is_mandatory"]),
            ", ".join(tag_info["tags"]),
            ", ".join(tag_info["used_by"]),
        ]
        table.append(row)
    return table


class OutputFormat(StrEnum):
    github = "github"
    plain = "plain"


BALSAMIC_COMMON_TAGS = copy.deepcopy(BALSAMIC_UMI_TAGS)

WORKFLOW_MAP = {
    Workflow.MIP_DNA: {
        "header": ["Mip-dna tags", "Mandatory", "HK tags", "Used by"],
        "tags": MIP_DNA_TAGS,
    },
    Workflow.MIP_RNA: {
        "header": ["Mip-rna tags", "Mandatory", "HK tags", "Used by"],
        "tags": MIP_RNA_TAGS,
    },
    Workflow.FLUFFY: {
        "header": ["Fluffy tags", "Mandatory", "HK tags", "Used by"],
        "tags": FLUFFY_COMMON_TAGS,
    },
    Workflow.MICROSALT: {
        "header": ["Microsalt tags", "Mandatory", "HK tags", "Used by"],
        "tags": MICROSALT_COMMON_TAGS,
    },
    Workflow.BALSAMIC: {
        "header": ["Balsamic tags", "Mandatory", "HK tags", "Used by"],
        "tags": BALSAMIC_COMMON_TAGS,
    },
    Workflow.MUTANT: {
        "header": ["Mutant tags", "Mandatory", "HK tags", "Used by"],
        "tags": MUTANT_COMMON_TAGS,
    },
    Workflow.RAREDISEASE: {
        "header": ["Raredisease tags", "Mandatory", "HK tags", "Used by"],
        "tags": RAREDISEASE_TAGS,
    },
    Workflow.RNAFUSION: {
        "header": ["Rnafusion tags", "Mandatory", "HK tags", "Used by"],
        "tags": RNAFUSION_TAGS,
    },
    Workflow.TAXPROFILER: {
        "header": ["Taxprofiler tags", "Mandatory", "HK tags", "Used by"],
        "tags": TAXPROFILER_TAGS,
    },
    Workflow.TOMTE: {
        "header": ["Tomte tags", "Mandatory", "HK tags", "Used by"],
        "tags": TOMTE_TAGS,
    },
}


@app.command(name="tags")
def export_tags_cmd(
    output: OutputFormat = typer.Option(OutputFormat.github),
    workflow: Workflow = None,
):
    """Export tag definitions from Hermes."""
    LOG.info(f"Running export tags for workflow: {workflow}")

    if not workflow:
        header = ["Tag name", "Description"]
        for category in COMMON_TAG_CATEGORIES:
            table_name: str = category.name()
            if output == "github":
                typer.echo(f"## {table_name}")
            else:
                typer.echo(table_name)
            typer.echo()
            table = [[tag.value, tag.description] for tag in category]
            typer.echo(tabulate(table, headers=header, tablefmt=output))
            typer.echo()
        raise typer.Exit()
    if workflow not in WORKFLOW_MAP:
        LOG.info("Could not recognize workflow")
        raise typer.Exit(code=1)

    header = WORKFLOW_MAP[workflow]["header"]
    table = get_table(WORKFLOW_MAP[workflow]["tags"])

    typer.echo(tabulate(table, headers=header, tablefmt=output))
